#include <vector>
#include <string>
#include "Rivet/Config/RivetCommon.hh"
#include "Rivet/Histo1D.hh"

namespace Rivet {

    Histo1D::Histo1D(size_t len_of_weightvec, const YODA::Scatter2D& refscatter, const string& path)
		: YODA::AnalysisObject("_RivetHisto1D", path) 
	{
        _persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(refscatter, path)));
		for (size_t i=1; i < len_of_weightvec; ++i) {
			// suffix path name with weightname
			string suffix = "[" + boost::lexical_cast<string>(i) + "]";
 			_persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(refscatter, path + suffix)));
		}

 		_active.reset();
	}

    Histo1D::Histo1D(size_t len_of_weightvec, const vector<double>& binedges, const string& path)
		: YODA::AnalysisObject("_RivetHisto1D", path)
	{
        _persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(binedges, path)));
		for (size_t i=1; i < len_of_weightvec; ++i) {
			// suffix path name with weightname
			string suffix = "[" + boost::lexical_cast<string>(i) + "]";
 			_persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(binedges, path + suffix)));
		}

 		_active.reset();
	}

    Histo1D::Histo1D(size_t len_of_weightvec, size_t nbins, double lower, double upper, const string& path) 
		: YODA::AnalysisObject("_RivetHisto1D", path) 
	{
        _persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(nbins, lower, upper, path)));
		for (size_t i=1; i < len_of_weightvec; ++i) {
			// suffix path name with weightname
			string suffix = "[" + boost::lexical_cast<string>(i) + "]";
 			_persistent.push_back(boost::make_shared<YODA::Histo1D>(YODA::Histo1D(nbins, lower, upper, path + suffix)));
		}

 		_active.reset();
	}

    void Histo1D::pushToPersistent(const vector<vector<double> >& weight) {
        // loop over subevents
        for (size_t m = 0; m < _persistent.size(); ++m) {

            YODA::Histo1DPtr sum = boost::make_shared<YODA::Histo1D>(_evgroup[0]->clone());
            sum->scaleW( weight[0][m] );

            for (size_t n = 1; n < _evgroup.size(); ++n) {
                YODA::Histo1DPtr tmp = boost::make_shared<YODA::Histo1D>(_evgroup[n]->clone());
                tmp->scaleW( weight[n][m] );
                *sum += *tmp;
            }

            // sum typically only has one bin filled
            // do we really need to fill bin.size() times?
            bool filled_something = false;

            foreach(const YODA::HistoBin1D& b, sum->bins()) {
                if ( b.effNumEntries() != 0 && b.sumW() != 0 ) {
                    // @todo number of fills will be wrong
                    // needs to be xMid. xMean is not valid if bin contains neg.weights
                    _persistent[m]->fill(b.xMid(), b.sumW());
                    filled_something = true;
                }
            }
            // dummy fill to ensure that
            // each _persistent gets filled at least once per event group
            // @todo rethink this, current impl always messes up bin 0 fill counts
            
//            if (! filled_something) {
//                _persistent[m]->fill(sum->xMin(), 0.0);
//            }

            //TODO Overflow

        }

        _evgroup.clear();
        _active.reset();
    }

}
